import os
import argparse
import numpy as np
import glob
import subprocess
import seaborn as sns
import pandas as pd
import matplotlib.pyplot as plt



if __name__ == "__main__":
    parser = argparse.ArgumentParser("report")
    parser.add_argument("commit")
    args = parser.parse_args()
    results_folder = os.path.abspath(os.path.join(__file__, os.pardir, "results"))

    commit = args.commit
    def n_threads(filename):
        # Extracts '2742685_1_threads.csv'
        basename = os.path.basename(filename)
        return int(basename.split("_")[1])

    commit_result_folder = f"{results_folder}/{commit}"
    csv_bench_results = sorted(glob.glob(f"{commit_result_folder}*/*.csv"),
    key=n_threads)
    if len(csv_bench_results) == 0:
        raise RuntimeError(f"No results for commit {commit}")


    os.makedirs(commit_result_folder, exist_ok=True)

    df = pd.concat(map(pd.read_csv, csv_bench_results))
    df = df.drop(columns=["n_neighbors", "func"])

    cols = [
        "n_samples_train",
        "n_samples_test",
        "n_features",
        "leaf_size",
    ]

    # This creates a category used for grouping
    df['t'] = df.n_threads.apply(str)
    df_grouped = df.groupby(cols)

    for i, (vals, df_g) in enumerate(df_grouped):
        # 16:9 ratio
        fig = plt.figure(figsize=(24, 13.5))
        ax = plt.gca()
        splot = sns.barplot(
            y="t", x="throughput", hue="implementation", data=df_g, ax=ax
        )
        _ = ax.set_xlabel("Throughput (in GB/s)")
        _ = ax.set_ylabel("Number of threads")
        _ = ax.tick_params(labelrotation=45)

        # Adding the numerical values of "x" to bar
        for p in splot.patches:
            _ = splot.annotate(
                f"{p.get_width():.4e}",
                (p.get_width(), p.get_y() + p.get_height() / 2),
                ha="center",
                va="center",
                size=10,
                xytext=(0, -12),
                textcoords="offset points",
            )

        title = (
            f"KDTree.__init__@{commit} - "
            f"Euclidean Distance, dtype=np.float64, {df_g.trial.max() + 1} trials\n"
        )
        title += (
            "n_samples_train=%s - n_samples_test=%s - "
            "n_features=%s - leaf_size=%s"
        % vals
        )
        _ = fig.suptitle(title, fontsize=16)
        fig_file = f"{commit_result_folder}/{i}.pdf"
        plt.savefig(fig_file, bbox_inches="tight")
        print(f"{fig_file} written.")
